package org.jboss.wikitransform.docbook;

import org.apache.velocity.Template;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.VelocityEngine;
import org.apache.velocity.runtime.resource.loader.ClasspathResourceLoader;
import org.htmlparser.Node;
import org.htmlparser.Tag;
import org.htmlparser.filters.TagNameFilter;
import org.htmlparser.tags.LinkTag;
import org.htmlparser.util.NodeList;

import java.io.File;
import java.io.FileOutputStream;
import java.io.PrintStream;
import java.io.StringWriter;
import java.net.MalformedURLException;
import java.net.URL;

/**
 * @author <a href="mailto:bill@burkecentral.com">Bill Burke</a>
 * @version $Revision: 1 $
 */
public class Master extends Chapter
{
   private String title = "RESTEasy JAX-RS";
   private String subTitle = "RESTFul Web Service for Java";
   private String releaseInfo = "1.0";


   public Master(URL url)
   {
      super("main", url);
   }

   public String getTitle()
   {
      return title;
   }

   public void setTitle(String title)
   {
      this.title = title;
   }

   public String getSubTitle()
   {
      return subTitle;
   }

   public void setSubTitle(String subTitle)
   {
      this.subTitle = subTitle;
   }

   public String getReleaseInfo()
   {
      return releaseInfo;
   }

   public void setReleaseInfo(String releaseInfo)
   {
      this.releaseInfo = releaseInfo;
   }

   public void generate()
   {
      NodeList list = Util.getContent(url);
      createChapters(this, list);

   }

   private static void createChapters(Chapter parent, NodeList list)
   {
      Chapter last = null;
      for (Node node : list.extractAllNodesThatMatch(new TagNamesFilter("ul", "li")).toNodeArray())
      {
         Tag tag = (Tag) node;

         if (tag.getTagName().equalsIgnoreCase("li"))
         {

            NodeList links = tag.getChildren().extractAllNodesThatMatch(new TagNameFilter("a"));
            String chapterName = tag.toPlainTextString().trim();
            URL chapterUrl = null;
            for (int j = 0; j < links.size(); j++)
            {
               LinkTag link = (LinkTag) links.elementAt(j);
               String linkUrl = link.getLink();
               try
               {
                  chapterUrl = new URL(linkUrl);
               }
               catch (MalformedURLException e)
               {
                  throw new RuntimeException("url: " + linkUrl, e);
               }
               break; // only one
            }
            Chapter chapter = new Chapter(chapterName, chapterUrl);
            last = chapter;
            parent.getSubchapters().add(chapter);
         }
         else
         {
            Chapter next = last;
            if (next == null) next = parent;
            createChapters(next, tag.getChildren());
         }
      }
   }

   public static void main(String[] args) throws Exception
   {
      System.out.println(args.length);
      URL url = new URL("http://wiki.jboss.org/wiki/ResteasyJAXRSDocumentation");
      File baseDir = new File("/Users/billburke/jboss");
      String masterFileName = "resteasy-jaxrs";

      build(url, baseDir, masterFileName);

   }

   public static void build(URL url, File baseDir, String masterFileName)
           throws Exception
   {
      Master master = new Master(url);
      master.setFilename(masterFileName);
      master.generate();
      String masterXml = velocity(master, "org/jboss/wikitransform/docbook/templates/master.xml");
      String build = velocity(master, "org/jboss/wikitransform/docbook/templates/build.xml");


      File docbookDirectory = new File(baseDir, master.getFileName() + "-dockbook");
      docbookDirectory.mkdirs();

      File main = new File(docbookDirectory, "reference/en");
      File modulesDirectory = new File(main, "modules");
      modulesDirectory.mkdirs();
      new File(main, "images").mkdirs();

      File masterXmlFile = new File(main, "master.xml");
      FileOutputStream os = new FileOutputStream(masterXmlFile);
      os.write(masterXml.getBytes());
      os.close();

      File buildFile = new File(docbookDirectory, "build.xml");
      FileOutputStream bos = new FileOutputStream(buildFile);
      bos.write(build.getBytes());
      bos.close();

      for (Chapter chapter : master.getSubchapters())
      {
         File chapterFile = new File(main, "modules/" + chapter.getFileName() + ".xml");
         FileOutputStream cos = new FileOutputStream(chapterFile);
         PrintStream ps = new PrintStream(cos);
         chapter.generate(ps, 0);


         cos.close();
      }
   }

   public static String velocity(Chapter master, String templateLocation) throws Exception
   {
      VelocityEngine ve = new VelocityEngine();
      ve.setProperty("resource.loader", "class");
      ve.setProperty("class.resource.loader.class", ClasspathResourceLoader.class.getName());
      ve.init();
      Template t = ve.getTemplate(templateLocation);
      VelocityContext context = new VelocityContext();
      context.put("master", master);

      StringWriter writer = new StringWriter();
      t.merge(context, writer);
      return writer.toString();

   }

}
