package org.jboss.wikitransform.docbook;

import org.htmlparser.Node;
import org.htmlparser.NodeFilter;
import org.htmlparser.Parser;
import org.htmlparser.Tag;
import org.htmlparser.lexer.Lexer;
import org.htmlparser.util.NodeList;
import org.htmlparser.util.ParserException;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;

/**
 * @author <a href="mailto:bill@burkecentral.com">Bill Burke</a>
 * @version $Revision: 1 $
 */
public class Util
{
   public static NodeList getContent(URL url)
   {
      try
      {
         String page = getPage(url);
         Lexer lexer = new Lexer(page);

         Parser parser = new Parser(lexer);
         NodeList list = parser.parse(null).elementAt(0).getChildren();
         list.keepAllNodesThatMatch(new NodeFilter()
         {
            public boolean accept(Node node)
            {
               if (!(node instanceof Tag)) return true;
               Tag tag = (Tag) node;
               if (!tag.getTagName().equalsIgnoreCase("a")) return true;
               if (tag.getChildren().toHtml().equals("?")) return false;
               return true;
            }
         }, true);

         return list;
      }
      catch (ParserException e)
      {
         throw new RuntimeException(e);
      }
   }

   public static String getPage(URL url)
   {
      try
      {
         InputStream is = url.openStream();

         byte[] bytes = readFromStream(1024, is);
         String page = new String(bytes);
         String begin = "<span class=\"wikiContent\">";
         String end = "<h3>Other languages:</h3>";

         page = page.substring(page.indexOf(begin) + begin.length());
         try
         {
            page = page.substring(0, page.indexOf(end));
         }
         catch (Exception e)
         {
            throw new RuntimeException(url + " problems with", e);
         }

         page = "<body>" + page + "</body>";
         is.close();
         return page;
      }
      catch (IOException e)
      {
         throw new RuntimeException(e);
      }
   }

   public static byte[] readFromStream(int bufferSize, InputStream entityStream)
           throws IOException
   {
      ByteArrayOutputStream baos = new ByteArrayOutputStream();

      byte[] buffer = new byte[bufferSize];
      int wasRead = 0;
      do
      {
         wasRead = entityStream.read(buffer);
         if (wasRead > 0)
         {
            baos.write(buffer, 0, wasRead);
         }
      } while (wasRead > -1);
      return baos.toByteArray();
   }

}
